package circularlist;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class CircularList<E> {
	private List<E> elements;
	private int index = -1;

	@SafeVarargs
	public CircularList(E... elements) {
		this.elements = new ArrayList<E>(Arrays.asList(elements));
	}

	/** get the element at index i % size(). */
	public E get(int i) {
		return elements.get(i % elements.size());
	}

	/**
	 * return the first index of the object o in the circular list, -1 if the object
	 * is not present.
	 * 
	 * @throws ObjectMissingException
	 */
	public int indexOf(Object o) throws ObjectMissingException {
		int i = elements.indexOf(o);
		if (i == -1)
			throw new ObjectMissingException(o);
		return i;
	}

	/** number of elements contained in the circular list. */
	public int size() {
		return elements.size();
	}

	public E nextElement() {
		index = ++index % elements.size();
		return elements.get(index);
	}

	public boolean add(E element) {
		return elements.add(element);
	}
	
	public void add(int i, E element) {
		elements.add(i, element);
	}
	
	public void addAll(CircularList<? extends E> cl) {
		elements.addAll(cl.elements);
	}
	
	public String toString() {
		return "Circular list of size " + size();
	}

	public boolean equals(Object o) {
		if (!(o instanceof CircularList))
			return false;
		CircularList<?> cl = (CircularList<?>) o;
		return elements.equals(cl.elements);
	}

	public int hashCode() {
		return elements.hashCode();
	}
}